@@ -105,6 +105,9 @@ struct _GstQTDemux {
   GstSegment segment;
   GstEvent *pending_newsegment;
   gboolean upstream_newsegment;
+  gint64 seek_offset;
+  gint64 push_seek_start;
+  gint64 push_seek_stop;
 
 #if 0
   /* gst index support */
@@ -112,9 +115,6 @@ struct _GstQTDemux {
   gint index_id;
 #endif
 
-  gint64 requested_seek_time;
-  guint64 seek_offset;
-
   gboolean upstream_seekable;
   gint64 upstream_size;
 
