@@ -25,9 +25,7 @@
 #endif
 
 #include <gst/rtp/gstrtpbuffer.h>
-#include <gst/audio/audio.h>
 #include "gstrtpsbcdepay.h"
-#include "gstrtputils.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpsbcdepay_debug);
 #define GST_CAT_DEFAULT (rtpsbcdepay_debug)
@@ -53,28 +51,15 @@ GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
         "encoding-name = (string) SBC")
     );
 
-enum
-{
-  PROP_0,
-  PROP_IGNORE_TIMESTAMPS,
-  PROP_LAST
-};
-
-#define DEFAULT_IGNORE_TIMESTAMPS FALSE
-
 #define gst_rtp_sbc_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpSbcDepay, gst_rtp_sbc_depay, GST_TYPE_RTP_BASE_DEPAYLOAD);
 
-static void gst_rtp_sbc_depay_set_property (GObject * object,
-    guint prop_id, const GValue * value, GParamSpec * pspec);
-static void gst_rtp_sbc_depay_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec);
 static void gst_rtp_sbc_depay_finalize (GObject * object);
 
 static gboolean gst_rtp_sbc_depay_setcaps (GstRTPBaseDepayload * base,
     GstCaps * caps);
 static GstBuffer *gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base,
-    GstRTPBuffer * rtp);
+    GstBuffer * in);
 
 static void
 gst_rtp_sbc_depay_class_init (GstRtpSbcDepayClass * klass)
@@ -85,21 +70,14 @@ gst_rtp_sbc_depay_class_init (GstRtpSbcDepayClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = gst_rtp_sbc_depay_finalize;
-  gobject_class->set_property = gst_rtp_sbc_depay_set_property;
-  gobject_class->get_property = gst_rtp_sbc_depay_get_property;
-
-  g_object_class_install_property (gobject_class, PROP_IGNORE_TIMESTAMPS,
-      g_param_spec_boolean ("ignore-timestamps", "Ignore Timestamps",
-          "Various statistics", DEFAULT_IGNORE_TIMESTAMPS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstbasertpdepayload_class->set_caps = gst_rtp_sbc_depay_setcaps;
-  gstbasertpdepayload_class->process_rtp_packet = gst_rtp_sbc_depay_process;
+  gstbasertpdepayload_class->process = gst_rtp_sbc_depay_process;
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_sbc_depay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_sbc_depay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_sbc_depay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_sbc_depay_sink_template));
 
   GST_DEBUG_CATEGORY_INIT (rtpsbcdepay_debug, "rtpsbcdepay", 0,
       "SBC Audio RTP Depayloader");
@@ -115,9 +93,6 @@ static void
 gst_rtp_sbc_depay_init (GstRtpSbcDepay * rtpsbcdepay)
 {
   rtpsbcdepay->adapter = gst_adapter_new ();
-  rtpsbcdepay->stream_align =
-      gst_audio_stream_align_new (48000, 40 * GST_MSECOND, 1 * GST_SECOND);
-  rtpsbcdepay->ignore_timestamps = DEFAULT_IGNORE_TIMESTAMPS;
 }
 
 static void
@@ -125,44 +100,11 @@ gst_rtp_sbc_depay_finalize (GObject * object)
 {
   GstRtpSbcDepay *depay = GST_RTP_SBC_DEPAY (object);
 
-  gst_audio_stream_align_free (depay->stream_align);
   gst_object_unref (depay->adapter);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-gst_rtp_sbc_depay_set_property (GObject * object,
-    guint prop_id, const GValue * value, GParamSpec * pspec)
-{
-  GstRtpSbcDepay *depay = GST_RTP_SBC_DEPAY (object);
-
-  switch (prop_id) {
-    case PROP_IGNORE_TIMESTAMPS:
-      depay->ignore_timestamps = g_value_get_boolean (value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_sbc_depay_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec)
-{
-  GstRtpSbcDepay *depay = GST_RTP_SBC_DEPAY (object);
-
-  switch (prop_id) {
-    case PROP_IGNORE_TIMESTAMPS:
-      g_value_set_boolean (value, depay->ignore_timestamps);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 /* FIXME: This duplicates similar functionality rtpsbcpay, but there isn't a
  * simple way to consolidate the two. This is best done by moving the function
  * to the codec-utils library in gst-plugins-base when these elements move to
@@ -237,11 +179,6 @@ gst_rtp_sbc_depay_setcaps (GstRTPBaseDepayload * base, GstCaps * caps)
   }
 
   gst_caps_unref (outcaps);
-  if (oldcaps)
-    gst_caps_unref (oldcaps);
-
-  /* Reset when the caps are changing */
-  gst_audio_stream_align_set_rate (depay->stream_align, depay->rate);
 
   return TRUE;
 
@@ -252,51 +189,30 @@ bad_caps:
 }
 
 static GstBuffer *
-gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstRTPBuffer * rtp)
+gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstBuffer * in)
 {
   GstRtpSbcDepay *depay = GST_RTP_SBC_DEPAY (base);
   GstBuffer *data = NULL;
+  GstRTPBuffer rtp = { NULL };
 
   gboolean fragment, start, last;
   guint8 nframes;
   guint8 *payload;
   guint payload_len;
-  gint samples = 0;
 
-  GstClockTime timestamp;
+  gst_rtp_buffer_map (in, GST_MAP_READ, &rtp);
 
   GST_LOG_OBJECT (depay, "Got %" G_GSIZE_FORMAT " bytes",
-      gst_buffer_get_size (rtp->buffer));
+      gst_buffer_get_size (in));
 
-  if (gst_rtp_buffer_get_marker (rtp)) {
+  if (gst_rtp_buffer_get_marker (&rtp)) {
     /* Marker isn't supposed to be set */
     GST_WARNING_OBJECT (depay, "Marker bit was set");
     goto bad_packet;
   }
 
-  timestamp = GST_BUFFER_DTS_OR_PTS (rtp->buffer);
-  if (depay->ignore_timestamps && timestamp == GST_CLOCK_TIME_NONE) {
-    GstClockTime initial_timestamp;
-    guint64 n_samples;
-
-    initial_timestamp =
-        gst_audio_stream_align_get_timestamp_at_discont (depay->stream_align);
-    n_samples =
-        gst_audio_stream_align_get_samples_since_discont (depay->stream_align);
-
-    if (initial_timestamp == GST_CLOCK_TIME_NONE) {
-      GST_ERROR_OBJECT (depay,
-          "Can only ignore timestamps on streams without valid initial timestamp");
-      return NULL;
-    }
-
-    timestamp =
-        initial_timestamp + gst_util_uint64_scale (n_samples, GST_SECOND,
-        depay->rate);
-  }
-
-  payload = gst_rtp_buffer_get_payload (rtp);
-  payload_len = gst_rtp_buffer_get_payload_len (rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
+  payload_len = gst_rtp_buffer_get_payload_len (&rtp);
 
   fragment = payload[0] & 0x80;
   start = payload[0] & 0x40;
@@ -306,7 +222,7 @@ gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstRTPBuffer * rtp)
   payload += 1;
   payload_len -= 1;
 
-  data = gst_rtp_buffer_get_payload_subbuffer (rtp, 1, -1);
+  data = gst_rtp_buffer_get_payload_subbuffer (&rtp, 1, -1);
 
   if (fragment) {
     /* Got a packet with a fragment */
@@ -326,25 +242,14 @@ gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstRTPBuffer * rtp)
     gst_adapter_push (depay->adapter, data);
 
     if (last) {
-      gint framelen, samples;
-      guint8 header[4];
-
       data = gst_adapter_take_buffer (depay->adapter,
           gst_adapter_available (depay->adapter));
-      gst_rtp_drop_non_audio_meta (depay, data);
-
-      if (gst_buffer_extract (data, 0, &header, 4) != 4 ||
-          gst_rtp_sbc_depay_get_params (depay, header,
-              payload_len, &framelen, &samples) < 0) {
-        gst_buffer_unref (data);
-        goto bad_packet;
-      }
-    } else {
+    } else
       data = NULL;
-    }
+
   } else {
     /* !fragment */
-    gint framelen;
+    gint framelen, samples;
 
     GST_LOG_OBJECT (depay, "Got %d frames", nframes);
 
@@ -354,8 +259,6 @@ gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstRTPBuffer * rtp)
       goto bad_packet;
     }
 
-    samples *= nframes;
-
     GST_LOG_OBJECT (depay, "Got payload of %d", payload_len);
 
     if (nframes * framelen > (gint) payload_len) {
@@ -366,19 +269,8 @@ gst_rtp_sbc_depay_process (GstRTPBaseDepayload * base, GstRTPBuffer * rtp)
     }
   }
 
-  if (depay->ignore_timestamps && data) {
-    GstClockTime duration;
-
-    gst_audio_stream_align_process (depay->stream_align,
-        GST_BUFFER_IS_DISCONT (rtp->buffer), timestamp, samples, &timestamp,
-        &duration, NULL);
-
-    GST_BUFFER_PTS (data) = timestamp;
-    GST_BUFFER_DTS (data) = GST_CLOCK_TIME_NONE;
-    GST_BUFFER_DURATION (data) = duration;
-  }
-
 out:
+  gst_rtp_buffer_unmap (&rtp);
   return data;
 
 bad_packet:
