@@ -250,19 +250,6 @@ flacparse_suite (void)
   Suite *s = suite_create ("flacparse");
   TCase *tc_chain = tcase_create ("general");
 
-
-  /* init test context */
-  ctx_factory = "flacparse";
-  ctx_sink_template = &sinktemplate;
-  ctx_src_template = &srctemplate;
-  ctx_discard = 3;
-  ctx_headers[0].data = streaminfo_header;
-  ctx_headers[0].size = sizeof (streaminfo_header);
-  ctx_headers[1].data = comment_header;
-  ctx_headers[1].size = sizeof (comment_header);
-
-  /* custom offsets, and ts always repeatedly 0 */
-  ctx_no_metadata = TRUE;
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_parse_flac_normal);
   tcase_add_test (tc_chain, test_parse_flac_drain_single);
@@ -282,4 +269,32 @@ flacparse_suite (void)
  *   - Both push- and pull-modes need to be tested
  *      * Pull-mode & EOS
  */
-GST_CHECK_MAIN (flacparse);
+
+int
+main (int argc, char **argv)
+{
+  int nf;
+
+  Suite *s = flacparse_suite ();
+  SRunner *sr = srunner_create (s);
+
+  gst_check_init (&argc, &argv);
+
+  /* init test context */
+  ctx_factory = "flacparse";
+  ctx_sink_template = &sinktemplate;
+  ctx_src_template = &srctemplate;
+  ctx_discard = 3;
+  ctx_headers[0].data = streaminfo_header;
+  ctx_headers[0].size = sizeof (streaminfo_header);
+  ctx_headers[1].data = comment_header;
+  ctx_headers[1].size = sizeof (comment_header);
+  /* custom offsets, and ts always repeatedly 0 */
+  ctx_no_metadata = TRUE;
+
+  srunner_run_all (sr, CK_NORMAL);
+  nf = srunner_ntests_failed (sr);
+  srunner_free (sr);
+
+  return nf;
+}
